@@ -1,19 +1,25 @@
+import threading
+import time
 from typing import Any, Dict, List, Optional, Tuple, Union
 import logging
 from tqdm import tqdm
 from multiprocessing import Pool, cpu_count
 from functools import partial
-import time
 from pathlib import Path
 import json
 import hashlib
 
-from lyrics_transcriber.types import LyricsData, PhraseScore, AnchorSequence, GapSequence, ScoredAnchor, TranscriptionResult, Word
+from lyrics_transcriber.types import LyricsData, PhraseScore, PhraseType, AnchorSequence, GapSequence, ScoredAnchor, TranscriptionResult, Word
 from lyrics_transcriber.correction.phrase_analyzer import PhraseAnalyzer
 from lyrics_transcriber.correction.text_utils import clean_text
 from lyrics_transcriber.utils.word_utils import WordUtils
 
 
+class AnchorSequenceTimeoutError(Exception):
+    """Raised when anchor sequence computation exceeds timeout."""
+    pass
+
+
 class AnchorSequenceFinder:
     """Identifies and manages anchor sequences between transcribed and reference lyrics."""
 
@@ -22,10 +28,16 @@ class AnchorSequenceFinder:
         cache_dir: Union[str, Path],
         min_sequence_length: int = 3,
         min_sources: int = 1,
+        timeout_seconds: int = 600,  # 10 minutes default timeout
+        max_iterations_per_ngram: int = 1000,  # Maximum iterations for while loop
+        progress_check_interval: int = 50,  # Check progress every N iterations
         logger: Optional[logging.Logger] = None,
     ):
         self.min_sequence_length = min_sequence_length
         self.min_sources = min_sources
+        self.timeout_seconds = timeout_seconds
+        self.max_iterations_per_ngram = max_iterations_per_ngram
+        self.progress_check_interval = progress_check_interval
         self.logger = logger or logging.getLogger(__name__)
         self.phrase_analyzer = PhraseAnalyzer(logger=self.logger)
         self.used_positions = {}
@@ -33,7 +45,14 @@ class AnchorSequenceFinder:
         # Initialize cache directory
         self.cache_dir = Path(cache_dir)
         self.cache_dir.mkdir(parents=True, exist_ok=True)
-        self.logger.debug(f"Initialized AnchorSequenceFinder with cache dir: {self.cache_dir}")
+        self.logger.info(f"Initialized AnchorSequenceFinder with cache dir: {self.cache_dir}, timeout: {timeout_seconds}s")
+
+    def _check_timeout(self, start_time: float, operation_name: str = "operation"):
+        """Check if timeout has occurred and raise exception if so."""
+        if self.timeout_seconds > 0:
+            elapsed_time = time.time() - start_time
+            if elapsed_time > self.timeout_seconds:
+                raise AnchorSequenceTimeoutError(f"{operation_name} exceeded {self.timeout_seconds} seconds (elapsed: {elapsed_time:.1f}s)")
 
     def _clean_text(self, text: str) -> str:
         """Clean text by removing punctuation and normalizing whitespace."""
@@ -174,18 +193,55 @@ class AnchorSequenceFinder:
         ref_words: Dict[str, List[Word]],
         min_sources: int,
     ) -> List[AnchorSequence]:
-        """Process a single n-gram length to find matching sequences."""
+        """Process a single n-gram length to find matching sequences with timeout and early termination."""
+        self.logger.info(f"🔍 N-GRAM {n}: Starting processing with {len(trans_words)} transcription words")
+        self.logger.info(f"🔍 N-GRAM {n}: Reference sources: {list(ref_texts_clean.keys())}")
+        self.logger.info(f"🔍 N-GRAM {n}: Max iterations limit: {self.max_iterations_per_ngram}")
+        
         candidate_anchors = []
         used_positions = {source: set() for source in ref_texts_clean.keys()}
         used_trans_positions = set()
+        
+        iteration_count = 0
+        last_progress_check = 0
+        last_anchor_count = 0
+        stagnation_count = 0
+        
+        self.logger.debug(f"🔍 N-GRAM {n}: Processing n-gram length {n} with max {self.max_iterations_per_ngram} iterations")
 
+        # Generate n-grams from transcribed text once
+        trans_ngrams = self._find_ngrams(trans_words, n)
+        self.logger.info(f"🔍 N-GRAM {n}: Generated {len(trans_ngrams)} n-grams for processing")
+
+        # Process all n-grams efficiently in multiple passes
         found_new_match = True
-        while found_new_match:
+        while found_new_match and iteration_count < self.max_iterations_per_ngram:
             found_new_match = False
+            iteration_count += 1
+            anchors_found_this_iteration = 0
 
-            # Generate n-grams from transcribed text
-            trans_ngrams = self._find_ngrams(trans_words, n)
+            # Log every 10th iteration to track progress
+            if iteration_count % 10 == 0:
+                self.logger.debug(f"🔍 N-GRAM {n}: Iteration {iteration_count}, anchors found: {len(candidate_anchors)}")
 
+            # Check for progress stagnation every N iterations
+            if iteration_count - last_progress_check >= self.progress_check_interval:
+                current_anchor_count = len(candidate_anchors)
+                if current_anchor_count == last_anchor_count:
+                    stagnation_count += 1
+                    self.logger.debug(f"🔍 N-GRAM {n}: Stagnation check {stagnation_count}/3 at iteration {iteration_count}")
+                    if stagnation_count >= 3:  # No progress for 3 consecutive checks
+                        self.logger.info(f"🔍 N-GRAM {n}: ⏹️ Early termination due to stagnation after {iteration_count} iterations")
+                        break
+                else:
+                    stagnation_count = 0  # Reset stagnation counter
+                
+                last_anchor_count = current_anchor_count
+                last_progress_check = iteration_count
+                
+                self.logger.debug(f"🔍 N-GRAM {n}: iteration {iteration_count}, anchors: {current_anchor_count}, stagnation: {stagnation_count}")
+
+            # Process all n-grams in this iteration
             for ngram, trans_pos in trans_ngrams:
                 # Skip if we've already used this transcription position
                 if trans_pos in used_trans_positions:
@@ -196,11 +252,11 @@ class AnchorSequenceFinder:
                 ngram_words = [w.lower() for w in ngram]
 
                 if actual_words != ngram_words:
-                    self.logger.error(f"Mismatch between ngram and actual words at position {trans_pos}:")
-                    self.logger.error(f"Ngram words: {ngram_words}")
-                    self.logger.error(f"Actual words: {actual_words}")
-                    self.logger.error(f"Full trans_words: {trans_words}")
-                    self.logger.error(f"Full all_words: {[w.text for w in all_words]}")
+                    self.logger.error(f"🔍 N-GRAM {n}: ❌ Mismatch between ngram and actual words at position {trans_pos}:")
+                    self.logger.error(f"🔍 N-GRAM {n}: Ngram words: {ngram_words}")
+                    self.logger.error(f"🔍 N-GRAM {n}: Actual words: {actual_words}")
+                    self.logger.error(f"🔍 N-GRAM {n}: Full trans_words: {trans_words}")
+                    self.logger.error(f"🔍 N-GRAM {n}: Full all_words: {[w.text for w in all_words]}")
                     raise AssertionError(
                         f"Ngram words don't match actual words at position {trans_pos}. "
                         f"This should never happen as trans_words should be derived from all_words."
@@ -208,6 +264,10 @@ class AnchorSequenceFinder:
 
                 matches = self._find_matching_sources(ngram, ref_texts_clean, n)
                 if len(matches) >= min_sources:
+                    # Log successful match
+                    if len(candidate_anchors) < 5:  # Only log first few matches to avoid spam
+                        self.logger.debug(f"🔍 N-GRAM {n}: ✅ Found match: '{' '.join(ngram)}' at pos {trans_pos} with {len(matches)} sources")
+                    
                     # Get Word IDs for transcribed words
                     transcribed_word_ids = [w.id for w in all_words[trans_pos : trans_pos + n]]
 
@@ -228,9 +288,26 @@ class AnchorSequenceFinder:
                         confidence=len(matches) / len(ref_texts_clean),
                     )
                     candidate_anchors.append(anchor)
+                    anchors_found_this_iteration += 1
                     found_new_match = True
-                    break
+                    
+                    # For efficiency, if we have very low iteration limits, find one match per iteration
+                    if self.max_iterations_per_ngram <= 10:
+                        break
+            
+            # Log progress for this iteration
+            if anchors_found_this_iteration > 0:
+                self.logger.debug(f"🔍 N-GRAM {n}: Found {anchors_found_this_iteration} anchors in iteration {iteration_count}")
+            
+            # Early termination if we've found enough anchors or processed all positions
+            if len(used_trans_positions) >= len(trans_ngrams) or len(candidate_anchors) >= len(trans_ngrams):
+                self.logger.info(f"🔍 N-GRAM {n}: ⏹️ Early termination - processed all positions after {iteration_count} iterations")
+                break
 
+        if iteration_count >= self.max_iterations_per_ngram:
+            self.logger.warning(f"🔍 N-GRAM {n}: ⏰ Processing terminated after reaching max iterations ({self.max_iterations_per_ngram})")
+        
+        self.logger.info(f"🔍 N-GRAM {n}: ✅ Completed processing after {iteration_count} iterations, found {len(candidate_anchors)} anchors")
         return candidate_anchors
 
     def find_anchors(
@@ -239,87 +316,229 @@ class AnchorSequenceFinder:
         references: Dict[str, LyricsData],
         transcription_result: TranscriptionResult,
     ) -> List[ScoredAnchor]:
-        """Find anchor sequences that appear in both transcription and references."""
-        cache_key = self._get_cache_key(transcribed, references, transcription_result)
-        cache_path = self.cache_dir / f"anchors_{cache_key}.json"
+        """Find anchor sequences that appear in both transcription and references with timeout protection."""
+        start_time = time.time()
+        
+        try:
+            self.logger.info(f"🔍 ANCHOR SEARCH: Starting find_anchors with timeout {self.timeout_seconds}s")
+            self.logger.info(f"🔍 ANCHOR SEARCH: Transcribed text length: {len(transcribed)}")
+            self.logger.info(f"🔍 ANCHOR SEARCH: Reference sources: {list(references.keys())}")
+            
+            cache_key = self._get_cache_key(transcribed, references, transcription_result)
+            cache_path = self.cache_dir / f"anchors_{cache_key}.json"
+            self.logger.info(f"🔍 ANCHOR SEARCH: Cache key: {cache_key}")
 
-        # Try to load from cache
-        if cached_data := self._load_from_cache(cache_path):
-            self.logger.info("Loading anchors from cache")
-            try:
-                # Convert cached_data to dictionary before logging
-                if cached_data:
-                    first_anchor = {"anchor": cached_data[0].anchor.to_dict(), "phrase_score": cached_data[0].phrase_score.to_dict()}
-                return cached_data
-            except Exception as e:
-                self.logger.error(f"Unexpected error loading cache: {type(e).__name__}: {e}")
-                if cached_data:
-                    try:
+            # Try to load from cache
+            self.logger.info(f"🔍 ANCHOR SEARCH: Checking cache at {cache_path}")
+            if cached_data := self._load_from_cache(cache_path):
+                self.logger.info("🔍 ANCHOR SEARCH: ✅ Cache hit! Loading anchors from cache")
+                try:
+                    # Convert cached_data to dictionary before logging
+                    if cached_data:
                         first_anchor = {"anchor": cached_data[0].anchor.to_dict(), "phrase_score": cached_data[0].phrase_score.to_dict()}
-                        self.logger.error(f"First cached anchor data: {json.dumps(first_anchor, indent=2)}")
-                    except:
-                        self.logger.error("Could not serialize first cached anchor for logging")
+                    return cached_data
+                except Exception as e:
+                    self.logger.error(f"🔍 ANCHOR SEARCH: ❌ Error loading cache: {type(e).__name__}: {e}")
+                    if cached_data:
+                        try:
+                            first_anchor = {"anchor": cached_data[0].anchor.to_dict(), "phrase_score": cached_data[0].phrase_score.to_dict()}
+                            self.logger.error(f"First cached anchor data: {json.dumps(first_anchor, indent=2)}")
+                        except:
+                            self.logger.error("Could not serialize first cached anchor for logging")
 
-        # If not in cache or cache format invalid, perform the computation
-        self.logger.info(f"Cache miss for key {cache_key} - computing anchors")
-        self.logger.info(f"Finding anchor sequences for transcription with length {len(transcribed)}")
+            # If not in cache or cache format invalid, perform the computation
+            self.logger.info(f"🔍 ANCHOR SEARCH: ❌ Cache miss - computing anchors with timeout {self.timeout_seconds}s")
+            self.logger.info(f"🔍 ANCHOR SEARCH: Finding anchor sequences for transcription with length {len(transcribed)}")
 
-        # Get all words from transcription
-        all_words = []
-        for segment in transcription_result.result.segments:
-            all_words.extend(segment.words)
+            # Check timeout before starting computation
+            self._check_timeout(start_time, "anchor computation initialization")
+            self.logger.info(f"🔍 ANCHOR SEARCH: ✅ Timeout check passed - initialization")
 
-        # Clean and split texts
-        trans_words = [w.text.lower().strip('.,?!"\n') for w in all_words]
-        ref_texts_clean = {
-            source: self._clean_text(" ".join(w.text for s in lyrics.segments for w in s.words)).split()
-            for source, lyrics in references.items()
-        }
-        ref_words = {source: [w for s in lyrics.segments for w in s.words] for source, lyrics in references.items()}
+            # Get all words from transcription
+            self.logger.info(f"🔍 ANCHOR SEARCH: Extracting words from transcription result...")
+            all_words = []
+            for segment in transcription_result.result.segments:
+                all_words.extend(segment.words)
+            self.logger.info(f"🔍 ANCHOR SEARCH: ✅ Extracted {len(all_words)} words from transcription")
 
-        # Filter out very short reference sources for n-gram length calculation
-        valid_ref_lengths = [
-            len(words) for words in ref_texts_clean.values()
-            if len(words) >= self.min_sequence_length
-        ]
+            # Clean and split texts
+            self.logger.info(f"🔍 ANCHOR SEARCH: Cleaning transcription words...")
+            trans_words = [w.text.lower().strip('.,?!"\n') for w in all_words]
+            self.logger.info(f"🔍 ANCHOR SEARCH: ✅ Cleaned {len(trans_words)} transcription words")
+            
+            self.logger.info(f"🔍 ANCHOR SEARCH: Processing reference sources...")
+            ref_texts_clean = {
+                source: self._clean_text(" ".join(w.text for s in lyrics.segments for w in s.words)).split()
+                for source, lyrics in references.items()
+            }
+            ref_words = {source: [w for s in lyrics.segments for w in s.words] for source, lyrics in references.items()}
+            
+            for source, words in ref_texts_clean.items():
+                self.logger.info(f"🔍 ANCHOR SEARCH: Reference '{source}': {len(words)} words")
 
-        if not valid_ref_lengths:
-            self.logger.warning("No reference sources long enough for anchor detection")
-            return []
+            # Check timeout after preprocessing
+            self._check_timeout(start_time, "anchor computation preprocessing")
+            self.logger.info(f"🔍 ANCHOR SEARCH: ✅ Timeout check passed - preprocessing")
 
-        # Calculate max length using only valid reference sources
-        max_length = min(len(trans_words), min(valid_ref_lengths))
-        n_gram_lengths = range(max_length, self.min_sequence_length - 1, -1)
+            # Filter out very short reference sources for n-gram length calculation
+            self.logger.info(f"🔍 ANCHOR SEARCH: Calculating n-gram lengths...")
+            valid_ref_lengths = [
+                len(words) for words in ref_texts_clean.values()
+                if len(words) >= self.min_sequence_length
+            ]
 
-        # Process n-gram lengths in parallel
-        process_length_partial = partial(
-            self._process_ngram_length,
-            trans_words=trans_words,
-            all_words=all_words,  # Pass the Word objects
-            ref_texts_clean=ref_texts_clean,
-            ref_words=ref_words,
-            min_sources=self.min_sources,
-        )
+            if not valid_ref_lengths:
+                self.logger.warning("🔍 ANCHOR SEARCH: ❌ No reference sources long enough for anchor detection")
+                return []
 
-        # Process n-gram lengths in parallel
-        candidate_anchors = []
-        with Pool(processes=max(cpu_count() - 1, 1)) as pool:
-            results = list(
-                tqdm(
-                    pool.imap(process_length_partial, n_gram_lengths, chunksize=1),
-                    total=len(n_gram_lengths),
-                    desc="Processing n-gram lengths",
-                )
+            # Calculate max length using only valid reference sources
+            max_length = min(len(trans_words), min(valid_ref_lengths))
+            n_gram_lengths = range(max_length, self.min_sequence_length - 1, -1)
+            self.logger.info(f"🔍 ANCHOR SEARCH: N-gram lengths to process: {list(n_gram_lengths)} (max_length: {max_length})")
+
+            # Process n-gram lengths in parallel with timeout
+            self.logger.info(f"🔍 ANCHOR SEARCH: Setting up parallel processing...")
+            process_length_partial = partial(
+                self._process_ngram_length,
+                trans_words=trans_words,
+                all_words=all_words,  # Pass the Word objects
+                ref_texts_clean=ref_texts_clean,
+                ref_words=ref_words,
+                min_sources=self.min_sources,
             )
-            for anchors in results:
-                candidate_anchors.extend(anchors)
 
-        self.logger.info(f"Found {len(candidate_anchors)} candidate anchors")
-        filtered_anchors = self._remove_overlapping_sequences(candidate_anchors, transcribed, transcription_result)
+            # Process n-gram lengths in parallel with timeout
+            candidate_anchors = []
+            pool_timeout = max(60, self.timeout_seconds // 2) if self.timeout_seconds > 0 else 300  # Use half the total timeout for pool operations
+            
+            # Check timeout before parallel processing
+            self._check_timeout(start_time, "parallel processing start")
+            self.logger.info(f"🔍 ANCHOR SEARCH: ✅ Timeout check passed - about to start parallel processing")
+            
+            try:
+                self.logger.info(f"🔍 ANCHOR SEARCH: 🚀 Starting parallel processing with {max(cpu_count() - 1, 1)} processes, pool timeout: {pool_timeout}s")
+                with Pool(processes=max(cpu_count() - 1, 1)) as pool:
+                    self.logger.debug(f"🔍 ANCHOR SEARCH: Pool created successfully")
+                    results = []
+                    
+                    # Submit all jobs first
+                    self.logger.info(f"🔍 ANCHOR SEARCH: Submitting {len(n_gram_lengths)} n-gram processing jobs...")
+                    async_results = []
+                    for i, n in enumerate(n_gram_lengths):
+                        self.logger.debug(f"🔍 ANCHOR SEARCH: Submitting job {i+1}/{len(n_gram_lengths)} for n-gram length {n}")
+                        async_result = pool.apply_async(process_length_partial, (n,))
+                        async_results.append(async_result)
+                    
+                    self.logger.info(f"🔍 ANCHOR SEARCH: ✅ All {len(async_results)} jobs submitted")
+                    
+                    # Collect results with individual timeouts
+                    batch_results = []
+                    batch_size = 10
+                    
+                    for i, async_result in enumerate(async_results):
+                        n_gram_length = n_gram_lengths[i]
+                        try:
+                            # Check remaining time for pool timeout (more lenient than overall timeout)
+                            elapsed_time = time.time() - start_time
+                            remaining_time = max(10, self.timeout_seconds - elapsed_time) if self.timeout_seconds > 0 else pool_timeout
+                            
+                            self.logger.debug(f"🔍 ANCHOR SEARCH: Remaining time for n-gram {n_gram_length}: {remaining_time}s")
+                            
+                            # Use a more lenient timeout for individual results to allow fallback
+                            individual_timeout = min(pool_timeout, remaining_time) if self.timeout_seconds > 0 else pool_timeout
+                            
+                            result = async_result.get(timeout=individual_timeout)
+                            results.append(result)
+                            
+                            # Batch logging - collect info for batched logging
+                            batch_results.append((n_gram_length, len(result)))
+                            
+                            # Log progress every batch_size results or on the last result
+                            if (i + 1) % batch_size == 0 or (i + 1) == len(async_results):
+                                total_anchors_in_batch = sum(anchor_count for _, anchor_count in batch_results)
+                                n_gram_ranges = [str(ng) for ng, _ in batch_results]
+                                range_str = f"{n_gram_ranges[0]}-{n_gram_ranges[-1]}" if len(n_gram_ranges) > 1 else n_gram_ranges[0]
+                                self.logger.info(f"🔍 ANCHOR SEARCH: ✅ Completed n-gram lengths {range_str} ({i+1-len(batch_results)+1}-{i+1}/{len(async_results)}) - found {total_anchors_in_batch} anchors total")
+                                batch_results = []  # Reset batch
+                            
+                        except Exception as e:
+                            self.logger.warning(f"🔍 ANCHOR SEARCH: ⚠️ n-gram length {n_gram_length} failed or timed out: {str(e)}")
+                            results.append([])  # Add empty result to maintain order
+                            
+                            # Add failed result to batch for logging
+                            batch_results.append((n_gram_length, 0))
+                            
+                            # If we're running short on time, trigger fallback early
+                            if self.timeout_seconds > 0 and (time.time() - start_time) > (self.timeout_seconds * 0.8):
+                                self.logger.warning(f"🔍 ANCHOR SEARCH: ⚠️ Approaching timeout limit, triggering early fallback")
+                                # Raise exception to trigger fallback to sequential processing
+                                raise Exception("Parallel processing timeout, triggering fallback")
+                    
+                    self.logger.info(f"🔍 ANCHOR SEARCH: ✅ Parallel processing completed, combining results...")
+                    for anchors in results:
+                        candidate_anchors.extend(anchors)
+                        
+            except AnchorSequenceTimeoutError:
+                self.logger.error(f"🔍 ANCHOR SEARCH: ❌ Parallel processing timed out")
+                # Re-raise timeout errors
+                raise
+            except Exception as e:
+                self.logger.error(f"🔍 ANCHOR SEARCH: ❌ Parallel processing failed: {str(e)}")
+                # Fall back to sequential processing with timeout checks
+                self.logger.info("🔍 ANCHOR SEARCH: 🔄 Falling back to sequential processing")
+                for n in n_gram_lengths:
+                    try:
+                        # Check timeout more leniently during sequential processing
+                        if self.timeout_seconds > 0:
+                            elapsed_time = time.time() - start_time
+                            # Allow more time for sequential processing (up to 2x the original timeout)
+                            if elapsed_time > (self.timeout_seconds * 2.0):
+                                self.logger.warning(f"🔍 ANCHOR SEARCH: ⏰ Sequential processing timeout for n-gram {n}")
+                                break
+                        
+                        self.logger.info(f"🔍 ANCHOR SEARCH: 🔄 Sequential processing n-gram length {n}")
+                        
+                        anchors = self._process_ngram_length(
+                            n, trans_words, all_words, ref_texts_clean, ref_words, self.min_sources
+                        )
+                        candidate_anchors.extend(anchors)
+                        self.logger.info(f"🔍 ANCHOR SEARCH: ✅ Sequential n-gram {n} completed - found {len(anchors)} anchors")
+                    except Exception as e:
+                        self.logger.warning(f"🔍 ANCHOR SEARCH: ⚠️ Sequential processing failed for n-gram length {n}: {str(e)}")
+                        continue
 
-        # Save to cache
-        self._save_to_cache(cache_path, filtered_anchors)
-        return filtered_anchors
+            self.logger.info(f"🔍 ANCHOR SEARCH: ✅ Found {len(candidate_anchors)} candidate anchors in {time.time() - start_time:.1f}s")
+            
+            # Check timeout before expensive filtering operation
+            self._check_timeout(start_time, "overlap filtering start")
+            self.logger.info(f"🔍 ANCHOR SEARCH: 🔄 Starting overlap filtering...")
+            
+            filtered_anchors = self._remove_overlapping_sequences(candidate_anchors, transcribed, transcription_result)
+            self.logger.info(f"🔍 ANCHOR SEARCH: ✅ Filtering completed - {len(filtered_anchors)} final anchors")
+
+            # Save to cache
+            self.logger.info(f"🔍 ANCHOR SEARCH: 💾 Saving results to cache...")
+            self._save_to_cache(cache_path, filtered_anchors)
+            
+            total_time = time.time() - start_time
+            self.logger.info(f"🔍 ANCHOR SEARCH: 🎉 Anchor sequence computation completed successfully in {total_time:.1f}s")
+            
+            return filtered_anchors
+            
+        except AnchorSequenceTimeoutError:
+            elapsed_time = time.time() - start_time
+            self.logger.error(f"🔍 ANCHOR SEARCH: ⏰ TIMEOUT after {elapsed_time:.1f}s (limit: {self.timeout_seconds}s)")
+            raise
+        except Exception as e:
+            elapsed_time = time.time() - start_time
+            self.logger.error(f"🔍 ANCHOR SEARCH: ❌ FAILED after {elapsed_time:.1f}s: {str(e)}")
+            self.logger.error(f"🔍 ANCHOR SEARCH: Exception type: {type(e).__name__}")
+            import traceback
+            self.logger.error(f"🔍 ANCHOR SEARCH: Traceback: {traceback.format_exc()}")
+            raise
+        finally:
+            # No cleanup needed for time-based timeout checks
+            pass
 
     def _score_sequence(self, words: List[str], context: str) -> PhraseScore:
         """Score a sequence based on its phrase quality"""
@@ -381,17 +600,21 @@ class AnchorSequenceFinder:
         context: str,
         transcription_result: TranscriptionResult,
     ) -> List[ScoredAnchor]:
-        """Remove overlapping sequences using phrase analysis."""
+        """Remove overlapping sequences using phrase analysis with timeout protection."""
+        self.logger.info(f"🔍 FILTERING: Starting overlap removal for {len(anchors)} anchors")
+        
         if not anchors:
+            self.logger.info(f"🔍 FILTERING: No anchors to process")
             return []
 
-        self.logger.info(f"Scoring {len(anchors)} anchors")
+        self.logger.info(f"🔍 FILTERING: Scoring {len(anchors)} anchors")
 
         # Create word map for scoring
         word_map = {w.id: w for s in transcription_result.result.segments for w in s.words}
+        self.logger.debug(f"🔍 FILTERING: Created word map with {len(word_map)} words")
 
         # Add word map to each anchor for scoring
-        for anchor in anchors:
+        for i, anchor in enumerate(anchors):
             # For backwards compatibility, only add transcribed_words if all IDs exist in word_map
             try:
                 anchor.transcribed_words = [word_map[word_id] for word_id in anchor.transcribed_word_ids]
@@ -429,32 +652,94 @@ class AnchorSequenceFinder:
 
         start_time = time.time()
 
-        # Try different pool sizes
+        # Try different pool sizes with timeout
         num_processes = max(cpu_count() - 1, 1)  # Leave one CPU free
-        self.logger.info(f"Using {num_processes} processes")
+        self.logger.info(f"🔍 FILTERING: Using {num_processes} processes for scoring")
 
         # Create a partial function with the context parameter fixed
         score_anchor_partial = partial(self._score_anchor_static, context=context)
 
-        # Use multiprocessing to score anchors in parallel
-        with Pool(processes=num_processes) as pool:
-            scored_anchors = list(
-                tqdm(
-                    pool.imap(score_anchor_partial, anchors, chunksize=50),
-                    total=len(anchors),
-                    desc="Scoring anchors (parallel)",
-                )
-            )
+        # Use multiprocessing to score anchors in parallel with timeout
+        scored_anchors = []
+        pool_timeout = 300  # 5 minutes for scoring phase
+        
+        try:
+            self.logger.info(f"🔍 FILTERING: 🚀 Starting parallel scoring with timeout {pool_timeout}s")
+            with Pool(processes=num_processes) as pool:
+                # Submit scoring jobs with timeout
+                async_results = []
+                batch_size = 50
+                
+                self.logger.info(f"🔍 FILTERING: Splitting {len(anchors)} anchors into batches of {batch_size}")
+                for i in range(0, len(anchors), batch_size):
+                    batch = anchors[i:i + batch_size]
+                    async_result = pool.apply_async(self._score_batch_static, (batch, context))
+                    async_results.append(async_result)
+                
+                self.logger.info(f"🔍 FILTERING: Submitted {len(async_results)} scoring batches")
+                
+                # Collect results with timeout
+                for i, async_result in enumerate(async_results):
+                    try:
+                        self.logger.debug(f"🔍 FILTERING: ⏳ Collecting batch {i+1}/{len(async_results)}")
+                        batch_results = async_result.get(timeout=pool_timeout)
+                        scored_anchors.extend(batch_results)
+                        self.logger.debug(f"🔍 FILTERING: ✅ Completed scoring batch {i+1}/{len(async_results)}")
+                    except Exception as e:
+                        self.logger.warning(f"🔍 FILTERING: ⚠️ Scoring batch {i+1} failed or timed out: {str(e)}")
+                        # Add basic scores for failed batch
+                        start_idx = i * batch_size
+                        end_idx = min((i + 1) * batch_size, len(anchors))
+                        for j in range(start_idx, end_idx):
+                            if j < len(anchors):
+                                try:
+                                    phrase_score = PhraseScore(
+                                        total_score=1.0,
+                                        natural_break_score=1.0,
+                                        phrase_type=PhraseType.COMPLETE
+                                    )
+                                    scored_anchors.append(ScoredAnchor(anchor=anchors[j], phrase_score=phrase_score))
+                                except:
+                                    continue
+                        
+        except Exception as e:
+            self.logger.warning(f"🔍 FILTERING: ❌ Parallel scoring failed: {str(e)}, falling back to basic scoring")
+            # Fall back to basic scoring
+            for anchor in anchors:
+                try:
+                    phrase_score = PhraseScore(
+                        total_score=1.0,
+                        natural_break_score=1.0,
+                        phrase_type=PhraseType.COMPLETE
+                    )
+                    scored_anchors.append(ScoredAnchor(anchor=anchor, phrase_score=phrase_score))
+                except:
+                    continue
 
         parallel_time = time.time() - start_time
-        self.logger.info(f"Parallel scoring took {parallel_time:.2f} seconds")
+        self.logger.info(f"🔍 FILTERING: ✅ Parallel scoring completed in {parallel_time:.2f}s, scored {len(scored_anchors)} anchors")
 
         # Sort and filter as before
+        self.logger.info(f"🔍 FILTERING: 🔄 Sorting anchors by priority...")
         scored_anchors.sort(key=self._get_sequence_priority, reverse=True)
+        self.logger.info(f"🔍 FILTERING: ✅ Sorting completed")
 
-        self.logger.info(f"Filtering {len(scored_anchors)} overlapping sequences")
+        self.logger.info(f"🔍 FILTERING: 🔄 Filtering {len(scored_anchors)} overlapping sequences")
         filtered_scored = []
-        for scored_anchor in tqdm(scored_anchors, desc="Filtering overlaps"):
+        
+        for i, scored_anchor in enumerate(scored_anchors):
+            # Check timeout every 100 anchors using our timeout mechanism (more lenient)
+            if i % 100 == 0 and i > 0:
+                # Only check timeout if we're significantly over the limit
+                if self.timeout_seconds > 0:
+                    elapsed_time = time.time() - start_time
+                    # Use a more lenient timeout for filtering (allow 50% more time)
+                    if elapsed_time > (self.timeout_seconds * 1.5):
+                        self.logger.warning(f"🔍 FILTERING: ⏰ Filtering timed out, returning {len(filtered_scored)} anchors out of {len(scored_anchors)}")
+                        break
+                
+                self.logger.debug(f"🔍 FILTERING: Progress: {i}/{len(scored_anchors)} processed, {len(filtered_scored)} kept")
+            
             overlaps = False
             for existing in filtered_scored:
                 if self._sequences_overlap(scored_anchor.anchor, existing.anchor):
@@ -464,7 +749,7 @@ class AnchorSequenceFinder:
             if not overlaps:
                 filtered_scored.append(scored_anchor)
 
-        self.logger.info(f"Filtered down to {len(filtered_scored)} non-overlapping anchors")
+        self.logger.info(f"🔍 FILTERING: ✅ Filtering completed - kept {len(filtered_scored)} non-overlapping anchors out of {len(scored_anchors)}")
         return filtered_scored
 
     @staticmethod
@@ -481,6 +766,30 @@ class AnchorSequenceFinder:
         phrase_score = AnchorSequenceFinder._score_anchor_static._phrase_analyzer.score_phrase(words, context)
         return ScoredAnchor(anchor=anchor, phrase_score=phrase_score)
 
+    @staticmethod
+    def _score_batch_static(anchors: List[AnchorSequence], context: str) -> List[ScoredAnchor]:
+        """Score a batch of anchors for better timeout handling."""
+        # Create analyzer only once per process
+        if not hasattr(AnchorSequenceFinder._score_batch_static, "_phrase_analyzer"):
+            AnchorSequenceFinder._score_batch_static._phrase_analyzer = PhraseAnalyzer(logger=logging.getLogger(__name__))
+
+        scored_anchors = []
+        for anchor in anchors:
+            try:
+                words = [w.text for w in anchor.transcribed_words]
+                phrase_score = AnchorSequenceFinder._score_batch_static._phrase_analyzer.score_phrase(words, context)
+                scored_anchors.append(ScoredAnchor(anchor=anchor, phrase_score=phrase_score))
+            except Exception:
+                # Add basic score for failed anchor
+                phrase_score = PhraseScore(
+                    total_score=1.0,
+                    natural_break_score=1.0,
+                    phrase_type=PhraseType.COMPLETE
+                )
+                scored_anchors.append(ScoredAnchor(anchor=anchor, phrase_score=phrase_score))
+        
+        return scored_anchors
+
     def _get_reference_words(self, source: str, ref_words: List[str], start_pos: Optional[int], end_pos: Optional[int]) -> List[str]:
         """Get words from reference text between two positions.
 
