@@ -16,6 +16,7 @@ import TranscriptionView from './TranscriptionView'
 import { WordClickInfo, FlashType } from './shared/types'
 import EditModal from './EditModal'
 import ReviewChangesModal from './ReviewChangesModal'
+import ReplaceAllLyricsModal from './ReplaceAllLyricsModal'
 import {
     addSegmentBefore,
     splitSegment,
@@ -39,6 +40,8 @@ declare global {
     interface Window {
         toggleAudioPlayback?: () => void;
         seekAndPlayAudio?: (startTime: number) => void;
+        getAudioDuration?: () => number;
+        isAudioPlaying?: boolean;
     }
 }
 
@@ -193,6 +196,7 @@ interface MemoizedHeaderProps {
     onRedo: () => void
     canUndo: boolean
     canRedo: boolean
+    onUnCorrectAll: () => void
 }
 
 // Create a memoized Header component
@@ -216,7 +220,8 @@ const MemoizedHeader = memo(function MemoizedHeader({
     onUndo,
     onRedo,
     canUndo,
-    canRedo
+    canRedo,
+    onUnCorrectAll
 }: MemoizedHeaderProps) {
     return (
         <Header
@@ -240,6 +245,7 @@ const MemoizedHeader = memo(function MemoizedHeader({
             onRedo={onRedo}
             canUndo={canUndo}
             canRedo={canRedo}
+            onUnCorrectAll={onUnCorrectAll}
         />
     );
 });
@@ -265,11 +271,7 @@ export default function LyricsAnalyzer({ data: initialData, onFileLoad, apiClien
         index: number
         originalSegment: LyricsSegment
     } | null>(null)
-    const [isEditAllModalOpen, setIsEditAllModalOpen] = useState(false)
-    const [globalEditSegment, setGlobalEditSegment] = useState<LyricsSegment | null>(null)
-    const [originalGlobalSegment, setOriginalGlobalSegment] = useState<LyricsSegment | null>(null)
-    const [originalTranscribedGlobalSegment, setOriginalTranscribedGlobalSegment] = useState<LyricsSegment | null>(null)
-    const [isLoadingGlobalEdit, setIsLoadingGlobalEdit] = useState(false)
+    const [isReplaceAllLyricsModalOpen, setIsReplaceAllLyricsModalOpen] = useState(false)
     const [isReviewModalOpen, setIsReviewModalOpen] = useState(false)
     const [currentAudioTime, setCurrentAudioTime] = useState(0)
     const [isUpdatingHandlers, setIsUpdatingHandlers] = useState(false)
@@ -397,11 +399,11 @@ export default function LyricsAnalyzer({ data: initialData, onFileLoad, apiClien
             isReviewModalOpen ||
             isAddLyricsModalOpen ||
             isFindReplaceModalOpen ||
-            isEditAllModalOpen ||
+            isReplaceAllLyricsModalOpen ||
             isTimingOffsetModalOpen
         )
         setIsAnyModalOpen(modalOpen)
-    }, [modalContent, editModalSegment, isReviewModalOpen, isAddLyricsModalOpen, isFindReplaceModalOpen, isEditAllModalOpen, isTimingOffsetModalOpen])
+    }, [modalContent, editModalSegment, isReviewModalOpen, isAddLyricsModalOpen, isFindReplaceModalOpen, isReplaceAllLyricsModalOpen, isTimingOffsetModalOpen])
 
     // Calculate effective mode based on modifier key states
     const effectiveMode = isCtrlPressed ? 'delete_word' : (isShiftPressed ? 'highlight' : interactionMode)
@@ -783,212 +785,54 @@ export default function LyricsAnalyzer({ data: initialData, onFileLoad, apiClien
         updateDataWithHistory(newData, 'find/replace'); // Update history
     }
 
-    // Add handler for Edit All functionality
-    const handleEditAll = useCallback(() => {
-        console.log('EditAll - Starting process');
-
-        // Create empty placeholder segments to prevent the modal from closing
-        const placeholderSegment: LyricsSegment = {
-            id: 'loading-placeholder',
-            words: [],
-            text: '',
-            start_time: 0,
-            end_time: 1
-        };
-
-        // Set placeholder segments first
-        setGlobalEditSegment(placeholderSegment);
-        setOriginalGlobalSegment(placeholderSegment);
-
-        // Show loading state
-        setIsLoadingGlobalEdit(true);
-        console.log('EditAll - Set loading state to true');
-
-        // Open the modal with placeholder data
-        setIsEditAllModalOpen(true);
-        console.log('EditAll - Set modal open to true');
-
-        // Use requestAnimationFrame to ensure the modal with loading state is rendered
-        // before doing the expensive operation
-        requestAnimationFrame(() => {
-            console.log('EditAll - Inside requestAnimationFrame');
-
-            // Use setTimeout to allow the modal to render before doing the expensive operation
-            setTimeout(() => {
-                console.log('EditAll - Inside setTimeout, starting data processing');
-
-                try {
-                    console.time('EditAll - Data processing');
-
-                    // Create a combined segment with all words from all segments
-                    const allWords = data.corrected_segments.flatMap(segment => segment.words)
-                    console.log(`EditAll - Collected ${allWords.length} words from all segments`);
-
-                    // Sort words by start time to maintain chronological order
-                    const sortedWords = [...allWords].sort((a, b) => {
-                        const aTime = a.start_time ?? 0
-                        const bTime = b.start_time ?? 0
-                        return aTime - bTime
-                    })
-                    console.log('EditAll - Sorted words by start time');
-
-                    // Create a global segment containing all words
-                    const globalSegment: LyricsSegment = {
-                        id: 'global-edit',
-                        words: sortedWords,
-                        text: sortedWords.map(w => w.text).join(' '),
-                        start_time: sortedWords[0]?.start_time ?? null,
-                        end_time: sortedWords[sortedWords.length - 1]?.end_time ?? null
-                    }
-                    console.log('EditAll - Created global segment');
-
-                    // Store the original global segment for reset functionality
-                    setGlobalEditSegment(globalSegment)
-                    console.log('EditAll - Set global edit segment');
-
-                    setOriginalGlobalSegment(JSON.parse(JSON.stringify(globalSegment)))
-                    console.log('EditAll - Set original global segment');
-
-                    // Create the original transcribed global segment for Un-Correct functionality
-                    if (originalData.original_segments) {
-                        console.log('EditAll - Processing original segments for Un-Correct functionality');
-
-                        // Get all words from original segments
-                        const originalWords = originalData.original_segments.flatMap((segment: LyricsSegment) => segment.words)
-                        console.log(`EditAll - Collected ${originalWords.length} words from original segments`);
-
-                        // Sort words by start time
-                        const sortedOriginalWords = [...originalWords].sort((a, b) => {
-                            const aTime = a.start_time ?? 0
-                            const bTime = b.start_time ?? 0
-                            return aTime - bTime
-                        })
-                        console.log('EditAll - Sorted original words by start time');
-
-                        // Create the original transcribed global segment
-                        const originalTranscribedGlobal: LyricsSegment = {
-                            id: 'original-transcribed-global',
-                            words: sortedOriginalWords,
-                            text: sortedOriginalWords.map(w => w.text).join(' '),
-                            start_time: sortedOriginalWords[0]?.start_time ?? null,
-                            end_time: sortedOriginalWords[sortedOriginalWords.length - 1]?.end_time ?? null
-                        }
-                        console.log('EditAll - Created original transcribed global segment');
-
-                        setOriginalTranscribedGlobalSegment(originalTranscribedGlobal)
-                        console.log('EditAll - Set original transcribed global segment');
-                    } else {
-                        setOriginalTranscribedGlobalSegment(null)
-                        console.log('EditAll - No original segments found, set original transcribed global segment to null');
-                    }
-
-                    console.timeEnd('EditAll - Data processing');
-                } catch (error) {
-                    console.error('Error preparing global edit data:', error);
-                } finally {
-                    // Clear loading state
-                    console.log('EditAll - Finished processing, setting loading state to false');
-                    setIsLoadingGlobalEdit(false);
-                }
-            }, 100); // Small delay to allow the modal to render
-        });
-    }, [data.corrected_segments, originalData.original_segments])
-
-    // Handle saving the global edit
-    const handleSaveGlobalEdit = useCallback((updatedSegment: LyricsSegment) => {
-        console.log('Global Edit - Saving with new approach:', {
-            updatedSegmentId: updatedSegment.id,
-            wordCount: updatedSegment.words.length,
-            originalSegmentCount: data.corrected_segments.length,
-            originalTotalWordCount: data.corrected_segments.reduce((count, segment) => count + segment.words.length, 0)
-        })
-
-        // Get the updated words from the global segment
-        const updatedWords = updatedSegment.words
-
-        // Create a new array of segments with the same structure as the original
-        const updatedSegments = []
-        let wordIndex = 0
-
-        // Distribute words to segments based on the original segment sizes
-        for (const segment of data.corrected_segments) {
-            const originalWordCount = segment.words.length
-
-            // Get the words for this segment from the updated global segment
-            const segmentWords = []
-            const endIndex = Math.min(wordIndex + originalWordCount, updatedWords.length)
-
-            for (let i = wordIndex; i < endIndex; i++) {
-                segmentWords.push(updatedWords[i])
-            }
-
-            // Update the word index for the next segment
-            wordIndex = endIndex
-
-            // If we have words for this segment, create an updated segment
-            if (segmentWords.length > 0) {
-                // Recalculate segment start and end times
-                const validStartTimes = segmentWords.map(w => w.start_time).filter((t): t is number => t !== null)
-                const validEndTimes = segmentWords.map(w => w.end_time).filter((t): t is number => t !== null)
-
-                const segmentStartTime = validStartTimes.length > 0 ? Math.min(...validStartTimes) : null
-                const segmentEndTime = validEndTimes.length > 0 ? Math.max(...validEndTimes) : null
-
-                // Create the updated segment
-                updatedSegments.push({
-                    ...segment,
-                    words: segmentWords,
-                    text: segmentWords.map(w => w.text).join(' '),
-                    start_time: segmentStartTime,
-                    end_time: segmentEndTime
-                })
-            }
+    // Add handler for Un-Correct All functionality
+    const handleUnCorrectAll = useCallback(() => {
+        if (!originalData.original_segments) {
+            console.warn('No original segments available for un-correcting')
+            return
         }
 
-        // If there are any remaining words, add them to the last segment
-        if (wordIndex < updatedWords.length) {
-            const remainingWords = updatedWords.slice(wordIndex)
-            const lastSegment = updatedSegments[updatedSegments.length - 1]
+        if (window.confirm('Are you sure you want to revert all segments to their original transcribed state? This will undo all corrections made.')) {
+            console.log('Un-Correct All: Reverting all segments to original transcribed state', {
+                originalSegmentCount: originalData.original_segments.length,
+                currentSegmentCount: data.corrected_segments.length
+            })
 
-            // Combine the remaining words with the last segment
-            const combinedWords = [...lastSegment.words, ...remainingWords]
-
-            // Recalculate segment start and end times
-            const validStartTimes = combinedWords.map(w => w.start_time).filter((t): t is number => t !== null)
-            const validEndTimes = combinedWords.map(w => w.end_time).filter((t): t is number => t !== null)
-
-            const segmentStartTime = validStartTimes.length > 0 ? Math.min(...validStartTimes) : null
-            const segmentEndTime = validEndTimes.length > 0 ? Math.max(...validEndTimes) : null
-
-            // Update the last segment
-            updatedSegments[updatedSegments.length - 1] = {
-                ...lastSegment,
-                words: combinedWords,
-                text: combinedWords.map(w => w.text).join(' '),
-                start_time: segmentStartTime,
-                end_time: segmentEndTime
+            // Create new data with original segments as corrected segments
+            const newData: CorrectionData = {
+                ...data,
+                corrected_segments: JSON.parse(JSON.stringify(originalData.original_segments))
             }
-        }
 
-        console.log('Global Edit - Updated Segments with new approach:', {
-            segmentCount: updatedSegments.length,
-            firstSegmentWordCount: updatedSegments[0]?.words.length,
-            totalWordCount: updatedSegments.reduce((count, segment) => count + segment.words.length, 0),
-            originalTotalWordCount: data.corrected_segments.reduce((count, segment) => count + segment.words.length, 0)
+            updateDataWithHistory(newData, 'un-correct all segments')
+        }
+    }, [originalData.original_segments, data, updateDataWithHistory])
+
+    // Add handler for Replace All Lyrics functionality
+    const handleReplaceAllLyrics = useCallback(() => {
+        console.log('ReplaceAllLyrics - Opening modal')
+        setIsReplaceAllLyricsModalOpen(true)
+    }, [])
+
+    // Handle saving new segments from Replace All Lyrics
+    const handleSaveReplaceAllLyrics = useCallback((newSegments: LyricsSegment[]) => {
+        console.log('ReplaceAllLyrics - Saving new segments:', {
+            segmentCount: newSegments.length,
+            totalWords: newSegments.reduce((count, segment) => count + segment.words.length, 0)
         })
 
-        // Update the data with the new segments
+        // Create new data with the replaced segments
         const newData = {
             ...data,
-            corrected_segments: updatedSegments
-        };
-        updateDataWithHistory(newData, 'edit all'); // Update history
-
-        // Close the modal
-        setIsEditAllModalOpen(false)
-        setGlobalEditSegment(null)
+            corrected_segments: newSegments
+        }
+        
+        updateDataWithHistory(newData, 'replace all lyrics')
+        setIsReplaceAllLyricsModalOpen(false)
     }, [data, updateDataWithHistory])
 
+
+
     // Undo/Redo handlers
     const handleUndo = useCallback(() => {
         if (historyIndex > 0) {
@@ -1087,13 +931,14 @@ export default function LyricsAnalyzer({ data: initialData, onFileLoad, apiClien
                 isUpdatingHandlers={isUpdatingHandlers}
                 onHandlerClick={handleHandlerClick}
                 onFindReplace={() => setIsFindReplaceModalOpen(true)}
-                onEditAll={handleEditAll}
+                onEditAll={handleReplaceAllLyrics}
                 onTimingOffset={handleOpenTimingOffsetModal}
                 timingOffsetMs={timingOffsetMs}
                 onUndo={handleUndo}
                 onRedo={handleRedo}
                 canUndo={canUndo}
                 canRedo={canRedo}
+                onUnCorrectAll={handleUnCorrectAll}
             />
 
             <Grid container direction={isMobile ? 'column' : 'row'}>
@@ -1162,38 +1007,7 @@ export default function LyricsAnalyzer({ data: initialData, onFileLoad, apiClien
                 </Grid>
             </Grid>
 
-            <EditModal
-                open={isEditAllModalOpen}
-                onClose={() => {
-                    setIsEditAllModalOpen(false)
-                    setGlobalEditSegment(null)
-                    setOriginalGlobalSegment(null)
-                    setOriginalTranscribedGlobalSegment(null)
-                    handleSetModalSpacebarHandler(undefined)
-                }}
-                segment={globalEditSegment ? 
-                    timingOffsetMs !== 0 ? 
-                        applyOffsetToSegment(globalEditSegment, timingOffsetMs) : 
-                        globalEditSegment : 
-                    null}
-                segmentIndex={null}
-                originalSegment={originalGlobalSegment ? 
-                    timingOffsetMs !== 0 ? 
-                        applyOffsetToSegment(originalGlobalSegment, timingOffsetMs) : 
-                        originalGlobalSegment : 
-                    null}
-                onSave={handleSaveGlobalEdit}
-                onPlaySegment={handlePlaySegment}
-                currentTime={currentAudioTime}
-                setModalSpacebarHandler={handleSetModalSpacebarHandler}
-                originalTranscribedSegment={originalTranscribedGlobalSegment ? 
-                    timingOffsetMs !== 0 ? 
-                        applyOffsetToSegment(originalTranscribedGlobalSegment, timingOffsetMs) : 
-                        originalTranscribedGlobalSegment : 
-                    null}
-                isGlobal={true}
-                isLoading={isLoadingGlobalEdit}
-            />
+
 
             <EditModal
                 open={Boolean(editModalSegment)}
@@ -1266,6 +1080,15 @@ export default function LyricsAnalyzer({ data: initialData, onFileLoad, apiClien
                 currentOffset={timingOffsetMs}
                 onApply={handleApplyTimingOffset}
             />
+
+            <ReplaceAllLyricsModal
+                open={isReplaceAllLyricsModalOpen}
+                onClose={() => setIsReplaceAllLyricsModalOpen(false)}
+                onSave={handleSaveReplaceAllLyrics}
+                onPlaySegment={handlePlaySegment}
+                currentTime={currentAudioTime}
+                setModalSpacebarHandler={handleSetModalSpacebarHandler}
+            />
         </Box>
     )
 } 
\ No newline at end of file
