@@ -188,7 +188,7 @@ class CDGGenerator:
 
     def _compose_cdg(self, toml_file: str) -> None:
         """Compose CDG using KaraokeComposer."""
-        kc = KaraokeComposer.from_file(toml_file)
+        kc = KaraokeComposer.from_file(toml_file, logger=self.logger)
         kc.compose()
         # kc.create_mp4(height=1080, fps=30)
 
@@ -302,6 +302,7 @@ class CDGGenerator:
             "lead_in_duration",
             "lead_in_total",
             "title_artist_gap",
+            "title_top_padding",
             "intro_duration_seconds",
             "first_syllable_buffer_seconds",
             "outro_background",
@@ -313,6 +314,16 @@ class CDGGenerator:
             "outro_line1_line2_gap",
         }
 
+        optional_styles_with_defaults = {
+            "title_top_padding": 0,
+            # Any other optional parameters with their default values
+        }
+        
+        # Add any missing optional parameters with their default values
+        for key, default_value in optional_styles_with_defaults.items():
+            if key not in cdg_styles:
+                cdg_styles[key] = default_value
+        
         missing_styles = required_styles - set(cdg_styles.keys())
         if missing_styles:
             raise ValueError(f"Missing required style parameters: {', '.join(missing_styles)}")
@@ -414,6 +425,8 @@ class CDGGenerator:
             "title_screen_transition": cdg_styles["title_screen_transition"],
             "instrumentals": instrumentals,
             "intro_duration_seconds": cdg_styles["intro_duration_seconds"],
+            "title_top_padding": cdg_styles["title_top_padding"],
+            "title_artist_gap": cdg_styles["title_artist_gap"],
             "first_syllable_buffer_seconds": cdg_styles["first_syllable_buffer_seconds"],
             "outro_background": cdg_styles["outro_background"],
             "outro_transition": cdg_styles["outro_transition"],
